package vpc.cfg;

import vpc.tir.TIRExpr;
import vpc.tir.TIROperator;
import vpc.tir.TIRCall;
import vpc.tir.TIRLocal;
import vpc.tir.TIRConst;

import java.io.PrintWriter;

public class CfgDotPrinter
{
    public static final String NL = "\n";

    public static void print(PrintWriter out, String name, Cfg cfg, CfgNodeRenderer renderer)
    {
        out.println("digraph \"" + name + "\" {");
        out.println("  node [shape=box]");

        for (Cfg.Node n : cfg.nodes) {
            assert n.trans != null : n.index;

            StringBuilder label = new StringBuilder();

            label.append("[" + n.index + "]\n");

            for (String s : renderer.before(n)) {
                label.append(s).append('\n');
            }

            for (TIRExpr e : n.stmts) {
                if (e instanceof TIRLocal.Set || e instanceof TIRCall || e instanceof TIROperator) {
                    renderer.render(label, e);
                    label.append(NL);
                }
                else {
                    renderer.render(label, e);
                    label.append(NL);
                    //throw new AssertionError("bad type: " + e.getClass().getName());
                }
            }

            Cfg.Trans t = n.trans;
            if (t instanceof Cfg.Trans.Branch) {
                Cfg.Trans.Branch tb = (Cfg.Trans.Branch) t;
                label.append("if ");
                renderer.render(label, tb.condition);
                label.append(NL);
                out.println("  b" + n.index + " -> b" + tb.trueEdge.target.index + " [label=\"T\"]");
                out.println("  b" + n.index + " -> b" + tb.falseEdge.target.index + " [label=\"F\"]");
            }
            else if (t instanceof Cfg.Trans.Goto) {
                Cfg.Trans.Goto tg = (Cfg.Trans.Goto) t;
                out.println("  b" + n.index + " -> b" + tg.edge.target.index);
            }
            else if (t instanceof Cfg.Trans.Return) {
                Cfg.Trans.Return tr = (Cfg.Trans.Return) t;
                label.append("return");
                if (tr.value != null) {
                    label.append(' ');
                    renderer.render(label, tr.value);
                }
                label.append(NL);
            }
            else if (t instanceof Cfg.Trans.Throw) {
                Cfg.Trans.Throw tt = (Cfg.Trans.Throw) t;
                label.append("throw ").append(tt.exception.getName());
                label.append(NL);
            }
            else if (t instanceof Cfg.Trans.Switch) {
                Cfg.Trans.Switch ts = (Cfg.Trans.Switch) t;
                label.append("switch ");
                renderer.render(label, ts.expr);
                for (Cfg.Trans.Switch.Case c : ts.cases) {
                    StringBuilder edgeLabel = new StringBuilder();
                    String sep = "";
                    for (TIRConst.Value v : c.values) {
                        edgeLabel.append(sep); sep = ",";
                        renderer.render(edgeLabel, v);
                    }
                    out.println("  b" + n.index + " -> b" + c.edge.target.index + " [label=\"" + escape(edgeLabel.toString()) + "\"]");
                }
                if (ts.defEdge != null) {
                    out.println("  b" + n.index + " -> b" + ts.defEdge.target.index);
                }
            }
            else {
                throw new AssertionError("bad type: " + t.getClass().getName());
            }

            for (String s : renderer.after(n)) {
                label.append(s).append('\n');
            }

            out.println("  b" + n.index + " [label=\"" + escape(label.toString()) + "\"]");
        }

        out.println("}");
    }

    private static String escape(String s)
    {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            switch (c) {
                case '"': case '\\':
                    buf.append('\\');
                    buf.append(c);
                    break;
                case '\n':
                    buf.append("\\l");
                    break;
                case '\r':
                    // Throw away.
                    break;
                default:
                    buf.append(c);
                    break;
            }
        }
        return buf.toString();
    }
}
