/**
 * Copyright (c) 2007, Ben L. Titzer
 * See the file "license.txt" for details.
 *
 * Created Nov 28, 2007
 */
package vpc.core.virgil;

import vpc.core.Language;
import vpc.core.Program;
import static vpc.core.virgil.V2TypeSystem.isVoid;
import static vpc.core.virgil.V2TypeSystem.isComponent;
import static vpc.core.virgil.V2TypeSystem.isNull;
import vpc.core.base.*;
import vpc.core.types.*;
import vpc.util.Cache;

/**
 * The <code>V3Language</code> definition.
 *
 * @author Ben L. Titzer
 */
public class V3Language implements Language {
    public TypeSystem getTypeSystem(Program program) {
        V3TypeSystem typeSystem = new V3TypeSystem();
        typeSystem.registerBinOp(PrimBool.TYPE, new Capability.BinOp("&&", new PrimBool.AND()));
        typeSystem.registerBinOp(PrimBool.TYPE, new Capability.BinOp("||", new PrimBool.OR()));
        return typeSystem;
    }

    public void initializeTypeEnv(Program program, Cache<Type> typeCache, TypeEnv env) {
        env.bindTypeCon("int", PrimInt32.TYPECON);
        env.bindTypeCon("char", PrimChar.TYPECON);
        env.bindTypeCon("bool", PrimBool.TYPECON);
        env.bindTypeCon("void", PrimVoid.TYPECON);
        env.bindTypeCon("null", Reference.NULL_TYPECON);

        env.bindTypeCon("string", new TypeCon.Singleton(program.charArrayType));
        env.bindTypeCon("Array", VirgilArray.TYPECON);
        // bind the raw types.
        for ( int cntr = 1; cntr <= 64; cntr++ ) {
            env.bindTypeCon(Integer.toString(cntr), new TypeCon.Singleton(PrimRaw.getType(cntr)));
        }

        // register the System component.
        V3System system = new V3System(program);
        VirgilComponent systemC = system.component;
        env.bindTypeCon(systemC.getName(), systemC.getTypeCon());
        program.virgil.components.add(systemC.getName(), systemC);
        program.namespace.addDecl(systemC.getName(), systemC);
    }

    public String renderType(Type t) {
        return t.toString();
    }

    public String renderTypeCon(TypeCon tc, TypeRef... args) {
        return tc.render(args);
    }

    public boolean implicitLocalInitialization() {
        return true;
    }

    public boolean isVarType(Type t) {
        return !(isComponent(t) || isNull(t));
    }

    public boolean requiresProgramDecl() {
        return false;
    }

    public boolean adjustTupleParams() {
        return true;
    }

    public boolean canReturnVoid() {
        return true;
    }
}
