package vpc.ssa;

import vpc.cfg.CfgNodeRenderer;
import vpc.cfg.Cfg;
import vpc.tir.TIRExpr;
import vpc.tir.TIRLocal;
import vpc.core.types.Type;
import vpc.core.decl.Method;

import java.util.Arrays;

public class SsaCfgNodeRenderer extends CfgNodeRenderer
{
    public final Ssa ssa;
    public static final String VarNumberSep = ":";

    public SsaCfgNodeRenderer(Ssa ssa)
    {
        this.ssa = ssa;
    }

    public Iterable<String> before(Cfg.Node node)
    {
        Ssa.Phi[] phis = ssa.phis[node.index];
        String[] lines = new String[phis.length];
        int i = 0;
        for (Ssa.Phi phi : phis) {
            lines[i] = renderPhi(phi);
            i++;
        }
        return Arrays.asList(lines);
    }

    private static String renderPhi(Ssa.Phi phi)
    {
        StringBuilder buf = new StringBuilder();

        Ssa.Var v = phi.getVar();
        buf.append(v.base.getName()).append(VarNumberSep).append(v.number);
        buf.append(" <- PHI(");
        String sep = "";
        for (Ssa.Var arg : phi.args) {
            buf.append(sep); sep = ",";
            buf.append(arg.number);
        }
        buf.append(")");

        return buf.toString();
    }

    @Override
    public void render(StringBuilder buf, TIRExpr e, boolean needParens)
    {
        if (e instanceof TIRLocal.Get) {
            TIRLocal.Get g = (TIRLocal.Get) e;
            renderVarRef(buf, g.temp, ssa.gets.get(g));
        }
        else if (e instanceof TIRLocal.Set) {
            TIRLocal.Set s = (TIRLocal.Set) e;
            renderVarRef(buf, s.temp, ssa.sets.get(s));
            buf.append(" <- ");
            render(buf, s.value);
        }
        else {
            super.render(buf, e, needParens);
        }
    }

    private static void renderVarRef(StringBuilder buf, Method.Temporary<Type> temp, Ssa.Ref r)
    {
        buf.append(temp.getName()).append(VarNumberSep);
        if (r == null) {
            buf.append("?");
        }
        else {
            Ssa.Var v = r.getVar();
            assert v.base == temp;
            buf.append(v.number);
        }
    }
}
