/*
 * SPI (Serial peripheral interface) driver.
 * @author Akop Palyan
 */
component SPI
{

    // SPI Control Register
    field SPIE: 3 = 07;
    field SPE:  3 = 06;
    field DORD: 3 = 05;
    field MSTR: 3 = 04;
    field CPOL: 3 = 03;
    field CPHA: 3 = 02;
    field SPR:  3 = 01;

    // SPI Status Register
    field SPIF:  3 = 07;
    field WCOL:  3 = 06;
    field SPI2X: 3 = 00;

    field t_queue: Queue<char> = new Queue<char>(10);
    field r_queue: Queue<char> = new Queue<char>(10);

    field transmitting: boolean;
    field master: boolean;

    method startMaster() {
        transmitting = false;
        master = true;

        device.SPCR[SPE::int] = 0b1;
        device.SPCR[MSTR::int] = 0b1;
        device.SPCR[SPR::int] = 0b1;
        device.SPCR[SPIE::int] = 0b1;
     }

    method startSlave() {
        transmitting = false;
        master = false;

        device.SPCR[SPE::int] = 0b1;
        device.SPCR[SPIE::int] = 0b1;
    }

    method beginTransmit() {
        if (!transmitting) {
            device.SPDR = t_queue.dequeue();
	    transmitting = true;
        }
    }

    method interruptHandler() {
        r_queue.enqueue(device.SPDR::char);
        if (!t_queue.empty()) {
            device.SPDR = t_queue.dequeue();
        } else if (master) {
            transmitting = false;
        }
    }

    method isTransmitting(): boolean {
        return !t_queue.empty();
    }

    method write(buffer: char[], index: int, nbytes: int): int {
        local nenqueued: int = 0;
        local mask = MCU.disable();
        while (nbytes > 0 and t_queue.enqueue(buffer[index + nenqueued])) {
            nbytes = nbytes - 1;
            nenqueued = nenqueued + 1;
        }
        if (master and !t_queue.empty()) beginTransmit();
        MCU.restore(mask);
        return nenqueued;
    }

    method read(buffer: char[], index: int, nbytes: int): int {
	local ndequeued: int = 0;
        local mask = MCU.disable();
        while (nbytes > 0 and !r_queue.empty()) {
            buffer[index + ndequeued] = r_queue.dequeue();
	    nbytes = nbytes - 1;
	    ndequeued = ndequeued + 1;
        }
        MCU.restore(mask);
        return ndequeued;
    }
}
